@@ -4,26 +4,29 @@
 #include <primesieve.h>
 #include <inttypes.h>
 #include <stdio.h>
+#include <stdlib.h>
 
-int main()
+int main(int argc, char** argv)
 {
+  uint64_t limit = 10000000000ull;
+
+  if (argc > 1)
+    limit = atol(argv[1]);
+
   primesieve_iterator it;
   primesieve_init(&it);
 
+  /* indicate exact bounds to improve performance */
+  primesieve_skipto(&it, 0, limit);
+
   uint64_t sum = 0;
   uint64_t prime = 0;
 
   /* iterate over the primes below 10^9 */
-  while ((prime = primesieve_next_prime(&it)) < 1000000000ull)
+  while ((prime = primesieve_next_prime(&it)) <= limit)
     sum += prime;
 
-  printf("Sum of the primes below 10^9 = %" PRIu64 "\n", sum);
-
-  /* generate primes > 1000 */
-  primesieve_skipto(&it, 1000, 1100);
-
-  while ((prime = primesieve_next_prime(&it)) < 1100)
-    printf("%" PRIu64 "\n", prime);
+  printf("Sum of the primes <= %" PRIu64 " = %" PRIu64 "\n", limit, sum);
 
   primesieve_free_iterator(&it);
 
