@@ -20,36 +20,27 @@
 #include <primesieve/Erat.hpp>
 #include <primesieve/forward.hpp>
 #include <primesieve/littleendian_cast.hpp>
+#include <primesieve/macros.hpp>
 #include <primesieve/PreSieve.hpp>
 #include <primesieve/PrimeGenerator.hpp>
+#include <primesieve/macros.hpp>
 #include <primesieve/pmath.hpp>
+#include <primesieve/pod_vector.hpp>
 #include <primesieve/intrinsics.hpp>
-#include <primesieve/resizeUninitialized.hpp>
 #include <primesieve/SievingPrimes.hpp>
 
 #include <stdint.h>
 #include <algorithm>
-#include <array>
-#include <cassert>
-#include <vector>
+#include <limits>
 
-/// Enable AVX512 if primesieve is compiled using e.g.
-/// -march=native on an AVX512 capable CPU.
-#if !defined(DISABLE_AVX512) && \
-     defined(__AVX512F__) && \
-     defined(__AVX512BW__) && \
-     defined(__AVX512VBMI2__) && \
-     __has_include(<immintrin.h>)
-  #define ENABLE_AVX512
+#if defined(MULTIARCH_AVX512)
   #include <immintrin.h>
 #endif
 
-using std::size_t;
-
 namespace {
 
 /// First 128 primes
-const std::array<uint64_t, 128> smallPrimes =
+const primesieve::pod_array<uint64_t, 128> smallPrimes =
 {
     2,   3,   5,   7,  11,  13,  17,  19,  23,  29,
    31,  37,  41,  43,  47,  53,  59,  61,  67,  71,
@@ -67,7 +58,7 @@ const std::array<uint64_t, 128> smallPrimes =
 };
 
 /// Number of primes <= n
-const std::array<uint8_t, 720> primePi =
+const primesieve::pod_array<uint8_t, 720> primePi =
 {
     0,   0,   1,   2,   2,   3,   3,   4,   4,   4,   4,   5,   5,   6,   6,
     6,   6,   7,   7,   8,   8,   8,   8,   9,   9,   9,   9,   9,   9,  10,
@@ -123,8 +114,11 @@ const std::array<uint8_t, 720> primePi =
 
 namespace primesieve {
 
-PrimeGenerator::PrimeGenerator(uint64_t start, uint64_t stop) :
-  Erat(start, stop)
+PrimeGenerator::PrimeGenerator(uint64_t start,
+                               uint64_t stop,
+                               PreSieve& preSieve) :
+  Erat(start, stop),
+  preSieve_(preSieve)
 { }
 
 uint64_t PrimeGenerator::maxCachedPrime()
@@ -132,9 +126,9 @@ uint64_t PrimeGenerator::maxCachedPrime()
   return smallPrimes.back();
 }
 
-size_t PrimeGenerator::getStartIdx() const
+std::size_t PrimeGenerator::getStartIdx() const
 {
-  size_t startIdx = 0;
+  std::size_t startIdx = 0;
 
   if (start_ > 1)
     startIdx = primePi[start_ - 1];
@@ -142,9 +136,9 @@ size_t PrimeGenerator::getStartIdx() const
   return startIdx;
 }
 
-size_t PrimeGenerator::getStopIdx() const
+std::size_t PrimeGenerator::getStopIdx() const
 {
-  size_t stopIdx = 0;
+  std::size_t stopIdx = 0;
 
   if (stop_ < maxCachedPrime())
     stopIdx = primePi[stop_];
@@ -155,27 +149,44 @@ size_t PrimeGenerator::getStopIdx() const
 }
 
 /// Used by iterator::prev_prime()
-void PrimeGenerator::initPrevPrimes(std::vector<uint64_t>& primes,
-                                    size_t* size)
+void PrimeGenerator::initPrevPrimes(pod_vector<uint64_t>& primes,
+                                    std::size_t* size)
 {
-  size_t n = primeCountApprox(start_, stop_);
+  auto resize = [](pod_vector<uint64_t>& primes,
+                   std::size_t size)
+  {
+    // Avoids reallocation in fillPrevPrimes()
+    size += 64;
 
-  if (start_ > maxCachedPrime())
+    if (primes.empty())
+      primes.resize(size);
+    // When sieving backwards the number of primes inside [start, stop]
+    // slowly increases in each new segment as there are more small
+    // than large primes. Our new size has been calculated using
+    // primeCountApprox(start, stop) which is usually too large by 4%
+    // near 10^12 and by 2.5% near 10^19. Hence if the new size is less
+    // than 1% larger than the old size we do not increase the primes
+    // buffer as it will likely be large enough to fit all primes.
+    else if (size > primes.size() &&
+             (double) size / (double) primes.size() > 1.01)
+    {
+      // Prevent unnecessary copying when resizing
+      primes.clear();
+      primes.resize(size);
+    }
+  };
+
+  std::size_t pix = primeCountApprox(start_, stop_);
+
+  if (start_ <= maxCachedPrime())
   {
-    // +64 prevents reallocation in fillPrevPrimes()
-    resizeUninitialized(primes, n + 64);
-    *size = 0;
-  }
-  else
-  {
-    size_t a = getStartIdx();
-    size_t b = getStopIdx();
-    assert(a <= b);
+    std::size_t a = getStartIdx();
+    std::size_t b = getStopIdx();
+    ASSERT(a <= b);
+
     *size = (start_ <= 2) + b - a;
-
-    n = std::max(*size, n);
-    resizeUninitialized(primes, n + 64);
-    size_t i = 0;
+    resize(primes, std::max(*size, pix));
+    std::size_t i = 0;
 
     if (start_ <= 2)
       primes[i++] = 0;
@@ -184,45 +195,86 @@ void PrimeGenerator::initPrevPrimes(std::vector<uint64_t>& primes,
               smallPrimes.begin() + b,
               &primes[i]);
   }
+  else
+  {
+    resize(primes, pix);
+    *size = 0;
+  }
 
   initErat();
 }
 
 /// Used by iterator::next_prime()
-void PrimeGenerator::initNextPrimes(std::vector<uint64_t>& primes,
-                                    size_t* size)
+void PrimeGenerator::initNextPrimes(pod_vector<uint64_t>& primes,
+                                    std::size_t* size)
 {
-  // A buffer of 512 primes provides good
+  auto resize = [](pod_vector<uint64_t>& primes,
+                   std::size_t size)
+  {
+    if (size > primes.size())
+    {
+      // Prevent unnecessary copying when resizing
+      primes.clear();
+      primes.resize(size);
+    }
+  };
+
+  // A buffer of 1024 primes provides good
   // performance with little memory usage.
-  resizeUninitialized(primes, 512);
+  std::size_t maxSize = 1 << 10;
 
   if (start_ <= maxCachedPrime())
   {
-    size_t a = getStartIdx();
-    size_t b = getStopIdx();
-
+    std::size_t a = getStartIdx();
+    std::size_t b = getStopIdx();
     *size = b - a;
-    assert(*size <= primes.size());
 
+    if (stop_ < maxCachedPrime() + 2)
+      resize(primes, *size);
+    else
+    {
+      // +64 is needed because our fillNextPrimes()
+      // algorithm aborts as soon as there is not
+      // enough space to store 64 more primes.
+      std::size_t minSize = *size + 64;
+      std::size_t pix = primeCountApprox(start_, stop_) + 64;
+      pix = inBetween(minSize, pix, maxSize);
+      pix = std::max(*size, pix);
+      resize(primes, pix);
+    }
+
+    ASSERT(primes.size() >= *size);
     std::copy(smallPrimes.begin() + a,
               smallPrimes.begin() + b,
               primes.begin());
   }
+  else
+  {
+    // +64 is needed because our fillNextPrimes()
+    // algorithm aborts as soon as there is not
+    // enough space to store 64 more primes.
+    std::size_t minSize = 64;
+    std::size_t pix = primeCountApprox(start_, stop_) + 64;
+    pix = inBetween(minSize, pix, maxSize);
+    resize(primes, pix);
+  }
 
   initErat();
 }
 
 void PrimeGenerator::initErat()
 {
-  uint64_t startErat = maxCachedPrime() + 1;
+  ASSERT(maxCachedPrime() >= 5);
+  uint64_t startErat = maxCachedPrime() + 2;
   startErat = std::max(startErat, start_);
   isInit_ = true;
 
   if (startErat <= stop_)
   {
+    preSieve_.init(startErat, stop_);
     int sieveSize = get_sieve_size();
-    Erat::init(startErat, stop_, sieveSize, preSieve_);
-    sievingPrimes_.init(this, preSieve_);
+    Erat::init(startErat, stop_, sieveSize, preSieve_, memoryPool_);
+    sievingPrimes_.init(this, sieveSize, preSieve_, memoryPool_);
   }
 }
 
@@ -246,8 +298,8 @@ void PrimeGenerator::sieveSegment()
 }
 
 /// Used by iterator::prev_prime()
-bool PrimeGenerator::sievePrevPrimes(std::vector<uint64_t>& primes,
-                                     size_t* size)
+bool PrimeGenerator::sievePrevPrimes(pod_vector<uint64_t>& primes,
+                                     std::size_t* size)
 {
   if (!isInit_)
     initPrevPrimes(primes, size);
@@ -258,21 +310,20 @@ bool PrimeGenerator::sievePrevPrimes(std::vector<uint64_t>& primes,
     return true;
   }
 
+  // We have generated all primes inside [start, stop], we cannot
+  // generate more primes using this PrimeGenerator. Therefore we
+  // need to allocate a new PrimeGenerator in iterator.cpp.
   return false;
 }
 
 /// Used by iterator::next_prime()
-bool PrimeGenerator::sieveNextPrimes(std::vector<uint64_t>& primes,
-                                     size_t* size)
+bool PrimeGenerator::sieveNextPrimes(pod_vector<uint64_t>& primes,
+                                     std::size_t* size)
 {
   *size = 0;
 
   if (!isInit_)
-  {
     initNextPrimes(primes, size);
-    if (*size > 0)
-      return false;
-  }
 
   if (hasNextSegment())
   {
@@ -280,14 +331,18 @@ bool PrimeGenerator::sieveNextPrimes(std::vector<uint64_t>& primes,
     return true;
   }
 
-  // primesieve only supports primes < 2^64. In case the next
-  // prime would be > 2^64 we simply return UINT64_MAX.
-  if (stop_ >= std::numeric_limits<uint64_t>::max())
-  {
-    primes[0] = ~0ull;
-    *size = 1;
-  }
+  // We have generated all primes <= stop, we cannot generate
+  // more primes using this PrimeGenerator. Therefore we
+  // need to allocate a new PrimeGenerator in iterator.cpp.
+  if (stop_ < std::numeric_limits<uint64_t>::max())
+    return false;
 
+  // The next prime would be > 2^64, however primesieve only
+  // supports primes < 2^64. In this case we simply return
+  // UINT64_MAX to the user (instead of throwing an exception).
+  primes.clear();
+  primes.push_back(~0ull);
+  *size = primes.size();
   return false;
 }
 
@@ -298,29 +353,29 @@ bool PrimeGenerator::sieveNextPrimes(std::vector<uint64_t>& primes,
 /// over the primes inside [a, b] we need to generate new
 /// primes which incurs an initialization overhead of O(sqrt(n)).
 ///
-void PrimeGenerator::fillPrevPrimes(std::vector<uint64_t>& primes,
-                                    size_t* size)
+void PrimeGenerator::fillPrevPrimes(pod_vector<uint64_t>& primes,
+                                    std::size_t* size)
 {
   while (sievePrevPrimes(primes, size))
   {
     // Use local variables to prevent the compiler from
     // writing temporary results to memory.
-    size_t i = *size;
+    std::size_t i = *size;
     uint64_t low = low_;
     uint64_t sieveIdx = sieveIdx_;
-    uint64_t sieveSize = sieveSize_;
-    uint8_t* sieve = sieve_;
+    uint64_t sieveSize = sieve_.size();
+    uint8_t* sieve = sieve_.data();
 
     while (sieveIdx < sieveSize)
     {
       // Each loop iteration can generate up to 64 primes,
       // so we have to make sure there is enough space
       // left in the primes vector.
-      if (i + 64 > primes.size())
-        resizeUninitialized(primes, i + 64);
+      if_unlikely(i + 64 > primes.size())
+        primes.resize(i + 64);
 
       uint64_t bits = littleendian_cast<uint64_t>(&sieve[sieveIdx]);
-      size_t j = i;
+      std::size_t j = i;
       i += popcnt64(bits);
 
       do
@@ -343,7 +398,130 @@ void PrimeGenerator::fillPrevPrimes(std::vector<uint64_t>& primes,
   }
 }
 
-#if defined(ENABLE_AVX512)
+/// This method is used by iterator::next_prime().
+/// This method stores only the next few primes (~ 1000) in the
+/// primes vector. Also for iterator::next_prime() there is no
+/// recurring initialization overhead (unlike prev_prime()) for
+/// this reason iterator::next_prime() runs up to 2x faster
+/// than iterator::prev_prime().
+///
+#if defined(MULTIARCH)
+  __attribute__ ((target ("default")))
+#endif
+void PrimeGenerator::fillNextPrimes(pod_vector<uint64_t>& primes,
+                                    std::size_t* size)
+{
+  do
+  {
+    if (sieveIdx_ >= sieve_.size())
+      if (!sieveNextPrimes(primes, size))
+        return;
+
+    // Use local variables to prevent the compiler from
+    // writing temporary results to memory.
+    std::size_t i = *size;
+    std::size_t maxSize = primes.size();
+    ASSERT(i + 64 <= maxSize);
+    uint64_t low = low_;
+    uint64_t sieveIdx = sieveIdx_;
+    uint64_t sieveSize = sieve_.size();
+    uint8_t* sieve = sieve_.data();
+
+    // Fill the buffer with at least (maxSize - 64) primes.
+    // Each loop iteration can generate up to 64 primes
+    // so we have to stop generating primes once there is
+    // not enough space for 64 more primes.
+    do
+    {
+      uint64_t bits = littleendian_cast<uint64_t>(&sieve[sieveIdx]);
+      std::size_t j = i;
+      i += popcnt64(bits);
+
+      do
+      {
+        primes[j+0] = nextPrime(bits, low); bits &= bits - 1;
+        primes[j+1] = nextPrime(bits, low); bits &= bits - 1;
+        primes[j+2] = nextPrime(bits, low); bits &= bits - 1;
+        primes[j+3] = nextPrime(bits, low); bits &= bits - 1;
+        j += 4;
+      }
+      while (j < i);
+
+      low += 8 * 30;
+      sieveIdx += 8;
+    }
+    while (i <= maxSize - 64 &&
+           sieveIdx < sieveSize);
+
+    low_ = low;
+    sieveIdx_ = sieveIdx;
+    *size = i;
+  }
+  while (*size == 0);
+}
+
+#if defined(MULTIARCH_POPCNT_BMI)
+
+/// This algorithm is identical to the default fillNextPrimes()
+/// method, except that the POPCNT & BMI instruction sets (x64 CPUs)
+/// have been enabled which should provide up to 10% speedup.
+///
+__attribute__ ((target ("popcnt,bmi")))
+void PrimeGenerator::fillNextPrimes(pod_vector<uint64_t>& primes,
+                                    std::size_t* size)
+{
+  do
+  {
+    if (sieveIdx_ >= sieve_.size())
+      if (!sieveNextPrimes(primes, size))
+        return;
+
+    // Use local variables to prevent the compiler from
+    // writing temporary results to memory.
+    std::size_t i = *size;
+    std::size_t maxSize = primes.size();
+    ASSERT(i + 64 <= maxSize);
+    uint64_t low = low_;
+    uint64_t sieveIdx = sieveIdx_;
+    uint64_t sieveSize = sieve_.size();
+    uint8_t* sieve = sieve_.data();
+
+    // Fill the buffer with at least (maxSize - 64) primes.
+    // Each loop iteration can generate up to 64 primes
+    // so we have to stop generating primes once there is
+    // not enough space for 64 more primes.
+    do
+    {
+      uint64_t bits = littleendian_cast<uint64_t>(&sieve[sieveIdx]);
+      std::size_t j = i;
+      i += popcnt64(bits);
+
+      do
+      {
+        primes[j+0] = nextPrime(bits, low); bits &= bits - 1;
+        primes[j+1] = nextPrime(bits, low); bits &= bits - 1;
+        primes[j+2] = nextPrime(bits, low); bits &= bits - 1;
+        primes[j+3] = nextPrime(bits, low); bits &= bits - 1;
+        j += 4;
+      }
+      while (j < i);
+
+      low += 8 * 30;
+      sieveIdx += 8;
+    }
+    while (i <= maxSize - 64 &&
+           sieveIdx < sieveSize);
+
+    low_ = low;
+    sieveIdx_ = sieveIdx;
+    *size = i;
+  }
+  while (*size == 0);
+}
+
+#endif
+
+#if defined(MULTIARCH_AVX512)
 
 /// This algorithm converts 1 bits from the sieve array into primes
 /// using AVX512. The algorithm is a modified version of the AVX512
@@ -355,21 +533,28 @@ void PrimeGenerator::fillPrevPrimes(std::vector<uint64_t>& primes,
 /// distributed relatively evenly. While processing a 64-bit word
 /// from the sieve array there are if checks that skip to the next
 /// loop iteration once all 1 bits have been processed. In my
-/// benchmarks this algorithm ran about 5% faster than the default
+/// benchmarks this algorithm ran about 10% faster than the default
 /// fillNextPrimes() algorithm which uses __builtin_ctzll().
 ///
-void PrimeGenerator::fillNextPrimes(std::vector<uint64_t>& primes,
-                                    size_t* size)
+__attribute__ ((target ("avx512f,avx512vbmi,avx512vbmi2,popcnt")))
+void PrimeGenerator::fillNextPrimes(pod_vector<uint64_t>& primes,
+                                    std::size_t* size)
 {
   do
   {
-    if (sieveIdx_ >= sieveSize_)
+    if (sieveIdx_ >= sieve_.size())
       if (!sieveNextPrimes(primes, size))
         return;
 
-    *size = 0;
-    uint64_t maxSize = primes.size();
-    assert(primes.size() >= 64);
+    // Use local variables to prevent the compiler from
+    // writing temporary results to memory.
+    std::size_t i = *size;
+    std::size_t maxSize = primes.size();
+    ASSERT(i + 64 <= maxSize);
+    uint64_t low = low_;
+    uint64_t sieveIdx = sieveIdx_;
+    uint64_t sieveSize = sieve_.size();
+    uint8_t* sieve = sieve_.data();
 
     __m512i avxBitValues = _mm512_set_epi8(
       (char) 241, (char) 239, (char) 233, (char) 229,
@@ -399,148 +584,85 @@ void PrimeGenerator::fillNextPrimes(std::vector<uint64_t>& primes,
     __m512i bytes_48_to_55 = _mm512_setr_epi64(48, 49, 50, 51, 52, 53, 54, 55);
     __m512i bytes_56_to_63 = _mm512_setr_epi64(56, 57, 58, 59, 60, 61, 62, 63);
 
-    while (sieveIdx_ < sieveSize_)
+    while (sieveIdx < sieveSize)
     {
       // Each iteration processes 8 bytes from the sieve array
-      uint64_t bits64 = *(uint64_t*) &sieve_[sieveIdx_];
+      uint64_t bits64 = *(uint64_t*) &sieve[sieveIdx];
       uint64_t primeCount = popcnt64(bits64);
 
       // Prevent _mm512_storeu_si512() buffer overrun
-      if (*size + primeCount + (8 - primeCount % 8) >= maxSize)
+      if (i + primeCount >= maxSize - 7)
         break;
 
-      __m512i base = _mm512_set1_epi64(low_);
-      uint64_t* primes64 = &primes[*size];
+      __m512i base = _mm512_set1_epi64(low);
+      uint64_t* primes64 = &primes[i];
 
       // These variables are not used anymore during this
       // iteration, increment for next iteration.
-      *size += primeCount;
-      low_ += 8 * 30;
-      sieveIdx_ += 8;
+      i += primeCount;
+      low += 8 * 30;
+      sieveIdx += 8;
 
-      // Convert 1-bits to 0xff bytes
-      __m512i bytes64 = _mm512_maskz_set1_epi8(bits64, (char) 0xff);
+      // Convert 1 bits from the sieve array (bits64) into prime
+      // bit values (bytes) using the avxBitValues lookup table and
+      // move all non zero bytes (bit values) to the beginning.
+      __m512i bitValues = _mm512_maskz_compress_epi8(bits64, avxBitValues);
 
-      // Convert 0xff bytes into prime number offsets
-      // using the avxBitValues lookup table.
-      __m512i primeOffsets = _mm512_and_si512(bytes64, avxBitValues);
-
-      // Move all non zero bytes (prime offsets) to the beginning
-      primeOffsets = _mm512_maskz_compress_epi8(bits64, primeOffsets);
-
-      // Convert the first 8 bytes (prime offsets)
+      // Convert the first 8 bytes (prime bit values)
       // into eight 64-bit prime numbers.
-      __m512i vprimes0 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_0_to_7, primeOffsets);
+      __m512i vprimes0 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_0_to_7, bitValues);
       vprimes0 = _mm512_add_epi64(base, vprimes0);
       _mm512_storeu_si512(&primes64[0], vprimes0);
 
       if (primeCount <= 8)
         continue;
 
-      __m512i vprimes1 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_8_to_15, primeOffsets);
+      __m512i vprimes1 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_8_to_15, bitValues);
       vprimes1 = _mm512_add_epi64(base, vprimes1);
       _mm512_storeu_si512(&primes64[8], vprimes1);
 
       if (primeCount <= 16)
         continue;
 
-      __m512i vprimes2 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_16_to_23, primeOffsets);
+      __m512i vprimes2 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_16_to_23, bitValues);
       vprimes2 = _mm512_add_epi64(base, vprimes2);
       _mm512_storeu_si512(&primes64[16], vprimes2);
 
       if (primeCount <= 24)
         continue;
 
-      __m512i vprimes3 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_24_to_31, primeOffsets);
+      __m512i vprimes3 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_24_to_31, bitValues);
       vprimes3 = _mm512_add_epi64(base, vprimes3);
       _mm512_storeu_si512(&primes64[24], vprimes3);
 
       if (primeCount <= 32)
         continue;
 
-      __m512i vprimes4 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_32_to_39, primeOffsets);
+      __m512i vprimes4 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_32_to_39, bitValues);
       vprimes4 = _mm512_add_epi64(base, vprimes4);
       _mm512_storeu_si512(&primes64[32], vprimes4);
 
       if (primeCount <= 40)
         continue;
 
-      __m512i vprimes5 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_40_to_47, primeOffsets);
+      __m512i vprimes5 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_40_to_47, bitValues);
       vprimes5 = _mm512_add_epi64(base, vprimes5);
       _mm512_storeu_si512(&primes64[40], vprimes5);
 
       if (primeCount <= 48)
         continue;
 
-      __m512i vprimes6 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_48_to_55, primeOffsets);
+      __m512i vprimes6 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_48_to_55, bitValues);
       vprimes6 = _mm512_add_epi64(base, vprimes6);
       _mm512_storeu_si512(&primes64[48], vprimes6);
 
       if (primeCount <= 56)
         continue;
 
-      __m512i vprimes7 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_56_to_63, primeOffsets);
+      __m512i vprimes7 = _mm512_maskz_permutexvar_epi8(0x0101010101010101ull, bytes_56_to_63, bitValues);
       vprimes7 = _mm512_add_epi64(base, vprimes7);
       _mm512_storeu_si512(&primes64[56], vprimes7);
     }
-  }
-  while (*size == 0);
-}
-
-#else
-
-/// This method is used by iterator::next_prime().
-/// This method stores only the next few primes (~ 200) in the
-/// primes vector. Also for iterator::next_prime() there is no
-/// recurring initialization overhead (unlike prev_prime()) for
-/// this reason iterator::next_prime() runs up to 2x faster
-/// than iterator::prev_prime().
-///
-void PrimeGenerator::fillNextPrimes(std::vector<uint64_t>& primes,
-                                    size_t* size)
-{
-  do
-  {
-    if (sieveIdx_ >= sieveSize_)
-      if (!sieveNextPrimes(primes, size))
-        return;
-
-    // Use local variables to prevent the compiler from
-    // writing temporary results to memory.
-    size_t i = 0;
-    size_t maxSize = primes.size();
-    assert(maxSize >= 64);
-    uint64_t low = low_;
-    uint64_t sieveIdx = sieveIdx_;
-    uint64_t sieveSize = sieveSize_;
-    uint8_t* sieve = sieve_;
-
-    // Fill the buffer with at least (maxSize - 64) primes.
-    // Each loop iteration can generate up to 64 primes
-    // so we have to stop generating primes once there is
-    // not enough space for 64 more primes.
-    do
-    {
-      uint64_t bits = littleendian_cast<uint64_t>(&sieve[sieveIdx]);
-      size_t j = i;
-      i += popcnt64(bits);
-
-      do
-      {
-        assert(j + 4 < maxSize);
-        primes[j+0] = nextPrime(bits, low); bits &= bits - 1;
-        primes[j+1] = nextPrime(bits, low); bits &= bits - 1;
-        primes[j+2] = nextPrime(bits, low); bits &= bits - 1;
-        primes[j+3] = nextPrime(bits, low); bits &= bits - 1;
-        j += 4;
-      }
-      while (j < i);
-
-      low += 8 * 30;
-      sieveIdx += 8;
-    }
-    while (i <= maxSize - 64 &&
-           sieveIdx < sieveSize);
 
     low_ = low;
     sieveIdx_ = sieveIdx;
