@@ -13,13 +13,13 @@
 #include <primesieve/forward.hpp>
 #include <primesieve/PrimeSieve.hpp>
 #include <primesieve/ParallelSieve.hpp>
+#include <primesieve/CountPrintPrimes.hpp>
 #include <primesieve/pmath.hpp>
-#include <primesieve/PrintPrimes.hpp>
+#include <primesieve/pod_vector.hpp>
 #include <primesieve/PreSieve.hpp>
 
 #include <stdint.h>
 #include <algorithm>
-#include <array>
 #include <chrono>
 #include <iostream>
 
@@ -33,7 +33,7 @@ struct SmallPrime
   const char* str;
 };
 
-const std::array<SmallPrime, 8> smallPrimes
+const primesieve::pod_array<SmallPrime, 8> smallPrimes
 {{
   { 2,  2, 0, "2" },
   { 3,  3, 0, "3" },
@@ -62,8 +62,6 @@ PrimeSieve::PrimeSieve(ParallelSieve* parent) :
   parent_(parent)
 { }
 
-PrimeSieve::~PrimeSieve() = default;
-
 void PrimeSieve::reset()
 {
   counts_.fill(0);
@@ -143,7 +141,7 @@ uint64_t PrimeSieve::getDistance() const
 
 uint64_t PrimeSieve::getCount(int i) const
 {
-  return counts_.at(i);
+  return counts_[i];
 }
 
 counts_t& PrimeSieve::getCounts()
@@ -189,7 +187,7 @@ void PrimeSieve::setStop(uint64_t stop)
 /// Set the size of the sieve array in KiB (kibibyte)
 void PrimeSieve::setSieveSize(int sieveSize)
 {
-  sieveSize_ = inBetween(16, sieveSize, 4096);
+  sieveSize_ = inBetween(16, sieveSize, 8192);
   sieveSize_ = floorPow2(sieveSize_);
 }
 
@@ -294,8 +292,8 @@ void PrimeSieve::sieve()
 
   if (stop_ >= 7)
   {
-    PrintPrimes printPrimes(*this);
-    printPrimes.sieve();
+    CountPrintPrimes countPrintPrimes(*this);
+    countPrintPrimes.sieve();
   }
 
   auto t2 = std::chrono::system_clock::now();
